@@ -41,7 +41,7 @@ typedef struct
     gchar *device;
     gchar *source;
     gint dpi;
-    gchar *scan_mode;
+    ScanMode scan_mode;
     gint depth;
     gboolean multi_page;
 } ScanRequest;
@@ -222,7 +222,7 @@ poll_for_devices (Scanner *scanner)
 }
 
 
-static void
+static gboolean
 control_option (SANE_Handle handle, SANE_Int index, SANE_Action action, void *value)
 {
     SANE_Status status;
@@ -230,6 +230,8 @@ control_option (SANE_Handle handle, SANE_Int index, SANE_Action action, void *va
     status = sane_control_option (handle, index, action, value, NULL);
     if (status != SANE_STATUS_GOOD)
         g_warning ("Error setting control option: %s", sane_strstatus(status));
+
+    return status == SANE_STATUS_GOOD;
 }
 
 
@@ -275,21 +277,24 @@ set_fixed_option (SANE_Handle handle, const SANE_Option_Descriptor *option, SANE
 }
 
 
-static void
+static gboolean
 set_string_option (SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_Int option_index, const char *value)
 {
     char *string;
     gsize value_size, size;
+    gboolean result;
 
-    g_return_if_fail (option->type == SANE_TYPE_STRING);
+    g_return_val_if_fail (option->type == SANE_TYPE_STRING, FALSE);
     
     value_size = strlen (value) + 1;
     size = option->size > value_size ? option->size : value_size;
     string = g_malloc(sizeof(char) * size);
     strcpy (string, value);
     g_debug ("sane_control_option (%d, SANE_ACTION_SET_VALUE, \"%s\")", option_index, value);
-    control_option (handle, option_index, SANE_ACTION_SET_VALUE, string);
+    result = control_option (handle, option_index, SANE_ACTION_SET_VALUE, string);
     g_free (string);
+   
+    return result;
 }
 
 
@@ -589,8 +594,67 @@ scan_thread (Scanner *scanner)
                             set_int_option (handle, option, option_index, request->depth);
                     }
                     else if (strcmp (option->name, SANE_NAME_SCAN_MODE) == 0) {
-                        if (request->scan_mode)
-                            set_string_option (handle, option, option_index, request->scan_mode);
+                        /* The names of scan modes often used in drivers, as taken from the sane-backends source */
+                        const char *color_scan_modes[] =
+                        {
+                            SANE_VALUE_SCAN_MODE_COLOR,
+                            "Color",
+                            NULL
+                        };
+                        const char *gray_scan_modes[] =
+                        {
+                            SANE_VALUE_SCAN_MODE_GRAY,
+                            "Gray",
+                            "Grayscale",
+                            SANE_I18N ("Grayscale"),
+                            NULL
+                        };
+                        const char *lineart_scan_modes[] =
+                        {
+                            SANE_VALUE_SCAN_MODE_LINEART,
+                            "Lineart",
+                            "LineArt",
+                            SANE_I18N ("LineArt"),
+                            "Black & White",
+                            SANE_I18N ("Black & White"),
+                            "Binary",
+                            SANE_I18N ("Binary"),
+                            SANE_VALUE_SCAN_MODE_GRAY,
+                            "Gray",
+                            "Grayscale",
+                            SANE_I18N ("Grayscale"),
+                            NULL
+                        };
+                        int i;
+
+                        switch (request->scan_mode) {
+                        case SCAN_MODE_COLOR:
+                            for (i = 0; color_scan_modes[i] != NULL; i++) {
+                                if (set_string_option (handle, option, option_index, color_scan_modes[i]))
+                                    break;
+                            }
+                            if (color_scan_modes[i] == NULL)
+                                g_warning ("Unable to set Color mode, please file a bug");
+                            break;
+                        case SCAN_MODE_GRAY:
+                            for (i = 0; gray_scan_modes[i] != NULL; i++) {
+                                if (set_string_option (handle, option, option_index, gray_scan_modes[i]))
+                                    break;
+                            }
+                            if (gray_scan_modes[i] == NULL)
+                                g_warning ("Unable to set Gray mode, please file a bug");
+                            break;
+                        case SCAN_MODE_LINEART:
+                            for (i = 0; lineart_scan_modes[i] != NULL; i++) {
+                                if (set_string_option (handle, option, option_index, lineart_scan_modes[i]))
+                                    break;
+                            }
+                            if (lineart_scan_modes[i] == NULL)
+                                g_warning ("Unable to set Lineart mode, please file a bug");
+                            break;
+                        default:
+                            break;
+                        }
                     }
 
                     /* Test scanner options (hoping will not effect other scanners...) */
@@ -754,7 +818,6 @@ scan_thread (Scanner *scanner)
             data = NULL;
             g_free (request->device);
             g_free (request->source);
-            g_free (request->scan_mode);
             g_free (request);
             request = NULL;
             state = STATE_IDLE;
@@ -799,19 +862,7 @@ scanner_scan (Scanner *scanner, const char *device, const char *source,
     if (source)
         request->source = g_strdup (source);
     request->dpi = dpi;
-    switch (scan_mode) {
-    case SCAN_MODE_COLOR:
-        request->scan_mode = g_strdup (SANE_VALUE_SCAN_MODE_COLOR);
-        break;
-    case SCAN_MODE_GRAY:
-        request->scan_mode = g_strdup (SANE_VALUE_SCAN_MODE_GRAY);
-        break;
-    case SCAN_MODE_LINEART:
-        request->scan_mode = g_strdup (SANE_VALUE_SCAN_MODE_LINEART);
-        break;
-    default:
-        break;
-    }
+    request->scan_mode = scan_mode;
     request->depth = depth;
     request->multi_page = multi_page;
     g_async_queue_push (scanner->priv->scan_queue, request);
